@@ -11,6 +11,7 @@
 #define _SQ_PLUS_H_
 
 #include <stdlib.h>
+#include <string.h>
 
 #ifdef __APPLE__
   #include <malloc/malloc.h>
@@ -47,8 +48,15 @@
 #define SQ_CALL_RAISE_ERROR SQFalse
 #endif
 
-#include "squirrel.h"
-
+// this does the same as commenting out the "#ifdef _UNICODE"-stuff in our
+// bundled squirrel.h, but works also for system-squirrel
+#ifdef _UNICODE
+  #undef _UNICODE
+  #include "squirrel.h"
+  #define _UNICODE
+#else
+  #include "squirrel.h"
+#endif // _UNICODE
 // C::B patch: so it builds on 64bit, ecapsulate bool/int/float using Squirrel types (this patch applies everywhere, where threse types are used)
 typedef SQInteger BOOL_T;
 typedef SQInteger INT_T;
@@ -147,10 +155,10 @@ struct ScriptStringVar : ScriptStringVarBase {
     return safeStringCopy(s,_s.s,MaxLength);
   }
   bool operator == (const ScriptStringVar & _s) {
-    return _strcmp(s,_s.s) == 0;
+    return strcmp(s,_s.s) == 0;
   }
   bool compareCaseInsensitive(const ScriptStringVar & _s) {
-    return _stricmp(s,_s.s) == 0;
+    return strcasecmp(s,_s.s) == 0;
   }
 };
 
@@ -413,6 +421,7 @@ template<typename T>
 void BindConstant(SquirrelObject & so,T constant,const SQChar * scriptVarName) {
   validateConstantType(constant);
   VarRefPtr pvr = createVarRef(so,scriptVarName);
+  // C::B patch: Handler for newer compilers
 #if __cplusplus>=201103L
   static_assert(sizeof(constant)<=sizeof(void*), "using larger type");
 #endif
